<?php

/*
 * Copyright (C) 2007 - Marcus Lunzenauer <mlunzena@uos.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */


/**
 * Cache implementation using memcached.
 *
 * @package     studip
 * @subpackage  cache
 *
 * @author    mlunzena
 * @copyright (c) Authors
 * @version   $Id$
 */

class MemcachedCache implements StudipCache {


  private $memcache;


  function __construct($hostname, $port) {
    if (!extension_loaded('memcached')) {
      throw new Exception('Memcache extension missing.');
    }

    $this->memcache = new Memcached();

    $status = $this->memcache->addServer($hostname, $port);
    if (!$status) {
      throw new Exception('Could not add server.');
    }
  }


  /**
   * Expire item from the cache.
   *
   * Example:
   *
   *   # expires foo
   *   $cache->expire('foo');
   *
   * @param   string  a single key.
   *
   * @returns void
   *
   */
  function expire($arg) {
    $this->memcache->delete($arg);
  }


  /**
   * Retrieve item from the server.
   *
   * Example:
   *
   *   # reads foo
   *   $foo = $cache->reads('foo');
   *
   * @param   string  a single key
   *
   * @returns mixed  the previously stored data if an item with such a key
   *                 exists on the server or FALSE on failure.
   */
  function read($arg) {
    return $this->memcache->get($arg);
  }


  /**
   * Store data at the server.
   *
   * @param string   the item's key.
   * @param string   the item's content.
   * @param int      the item's expiry time in seconds. Defaults to 12h.
   *
   * @returns mixed  returns TRUE on success or FALSE on failure.
   *
   */
  function write($name, $content, $expire = 43200) {
    return $this->memcache->set($name, $content, $expire);
  }
}
